@@ -9,11 +9,10 @@
 """
 
 __author__ = "Ryan McGrath <ryan@venodesigns.net>"
-__version__ = "1.5.2"
+__version__ = "1.6.0"
 
 import urllib
 import re
-import inspect
 import time
 
 import requests
@@ -28,19 +27,21 @@ except ImportError:
 
 # Twython maps keyword based arguments to Twitter API endpoints. The endpoints
 # table is a file with a dictionary of every API endpoint that Twython supports.
-from twitter_endpoints import base_url, api_table
+from twitter_endpoints import base_url, api_table, twitter_http_status_codes
 
 
 # There are some special setups (like, oh, a Django application) where
 # simplejson exists behind the scenes anyway. Past Python 2.6, this should
 # never really cause any problems to begin with.
 try:
-    # Python 2.6 and up
-    import json as simplejson
+    # Python 2.6 and below (2.4/2.5, 2.3 is not guranteed to work with this library to begin with)
+    # If they have simplejson, we should try and load that first,
+    # if they have the library, chances are they're gonna want to use that.
+    import simplejson
 except ImportError:
     try:
-        # Python 2.6 and below (2.4/2.5, 2.3 is not guranteed to work with this library to begin with)
-        import simplejson
+        # Python 2.6 and up
+        import json as simplejson
     except ImportError:
         try:
             # This case gets rarer by the day, but if we need to, we can pull it from Django provided it's there.
@@ -49,20 +50,6 @@ except ImportError:
             # Seriously wtf is wrong with you if you get this Exception.
             raise Exception("Twython requires the simplejson library (or Python 2.6) to work. http://www.undefined.org/python/")
 
-# Try and gauge the old OAuth2 library spec. Versions 1.5 and greater no longer have the callback
-# url as part of the request object; older versions we need to patch for Python 2.5... ugh. ;P
-OAUTH_CALLBACK_IN_URL = False
-OAUTH_LIB_SUPPORTS_CALLBACK = False
-if not hasattr(oauth, '_version') or float(oauth._version.manual_verstr) <= 1.4:
-    OAUTH_CLIENT_INSPECTION = inspect.getargspec(oauth.Client.request)
-    try:
-        OAUTH_LIB_SUPPORTS_CALLBACK = 'callback_url' in OAUTH_CLIENT_INSPECTION.args
-    except AttributeError:
-        # Python 2.5 doesn't return named tuples, so don't look for an args section specifically.
-        OAUTH_LIB_SUPPORTS_CALLBACK = 'callback_url' in OAUTH_CLIENT_INSPECTION
-else:
-    OAUTH_CALLBACK_IN_URL = True
-
 
 class TwythonError(AttributeError):
     """
@@ -76,8 +63,15 @@ class TwythonError(AttributeError):
     """
     def __init__(self, msg, error_code=None):
         self.msg = msg
-        if error_code == 400:
-            raise TwythonAPILimit(msg)
+
+        if error_code is not None and error_code in twitter_http_status_codes:
+            self.msg = '%s: %s -- %s' % \
+                        (twitter_http_status_codes[error_code][0],
+                         twitter_http_status_codes[error_code][1],
+                         self.msg)
+
+        if error_code == 400 or error_code == 420:
+            raise TwythonAPILimit(self.msg)
 
     def __str__(self):
         return repr(self.msg)
@@ -175,6 +169,7 @@ class Twython(object):
         self.access_token_url = 'http://twitter.com/oauth/access_token'
         self.authorize_url = 'http://twitter.com/oauth/authorize'
         self.authenticate_url = 'http://twitter.com/oauth/authenticate'
+        self.api_url = 'http://api.twitter.com/%s/'
 
         self.twitter_token = twitter_token
         self.twitter_secret = twitter_secret
@@ -193,7 +188,7 @@ class Twython(object):
             self.client = requests.session(hooks={'pre_request': OAuthHook()})
 
         if self.oauth_token is not None and self.oauth_secret is not None:
-            self.oauth_hook = OAuthHook(self.oauth_token, self.oauth_secret)
+            self.oauth_hook = OAuthHook(self.oauth_token, self.oauth_secret, header_auth=True)
             self.client = requests.session(hooks={'pre_request': self.oauth_hook})
 
         # Filter down through the possibilities here - if they have a token, if they're first stage, etc.
@@ -207,6 +202,9 @@ class Twython(object):
         for key in api_table.keys():
             self.__dict__[key] = setFunc(key)
 
+        # create stash for last call intel
+        self._last_call = None
+
     def _constructFunc(self, api_call, **kwargs):
         # Go through and replace any mustaches that are in our API url.
         fn = api_table[api_call]
@@ -221,16 +219,114 @@ class Twython(object):
         if not method in ('get', 'post', 'delete'):
             raise TwythonError('Method must be of GET, POST or DELETE')
 
+        content = self._request(url, method=method, params=kwargs)
+
+        return content
+
+    def _request(self, url, method='GET', params=None, api_call=None):
+        '''
+        Internal response generator, not sense in repeating the same
+        code twice, right? ;)
+        '''
         myargs = {}
+        method = method.lower()
+
         if method == 'get':
-            url = '%s?%s' % (url, urllib.urlencode(kwargs))
+            url = '%s?%s' % (url, urllib.urlencode(params))
         else:
-            myargs = kwargs
+            myargs = params
 
         func = getattr(self.client, method)
         response = func(url, data=myargs)
+        content = response.content.decode('utf-8')
 
-        return simplejson.loads(response.content.decode('utf-8'))
+        # create stash for last function intel
+        self._last_call = {
+            'api_call': api_call,
+            'api_error': None,
+            'cookies': response.cookies,
+            'error': response.error,
+            'headers': response.headers,
+            'status_code': response.status_code,
+            'url': response.url,
+            'content': content,
+        }
+
+        # Python 2.6 `json` will throw a ValueError if it
+        # can't load the string as valid JSON,
+        # `simplejson` will throw simplejson.decoder.JSONDecodeError
+        # But excepting just ValueError will work with both. o.O
+        try:
+            content = simplejson.loads(content)
+        except ValueError:
+            raise TwythonError('Response was not valid JSON, unable to decode.')
+
+        if response.status_code > 304:
+            # Just incase there is no error message, let's set a default
+            error_msg = 'An error occurred processing your request.'
+            if content.get('error') is not None:
+                error_msg = content['error']
+
+            self._last_call = error_msg
+
+            raise TwythonError(error_msg, error_code=response.status_code)
+
+        return content
+
+    '''
+    # Dynamic Request Methods
+    Just in case Twitter releases something in their API
+    and a developer wants to implement it on their app, but
+    we haven't gotten around to putting it in Twython yet. :)
+    '''
+
+    def request(self, endpoint, method='GET', params=None, version=1):
+        params = params or {}
+
+        # In case they want to pass a full Twitter URL
+        # i.e. http://search.twitter.com/
+        if endpoint.startswith('http://'):
+            url = endpoint
+        else:
+            url = '%s%s.json' % (self.api_url % version, endpoint)
+
+        content = self._request(url, method=method, params=params, api_call=url)
+
+        return content
+
+    def get(self, endpoint, params=None, version=1):
+        params = params or {}
+        return self.request(endpoint, params=params, version=version)
+
+    def post(self, endpoint, params=None, version=1):
+        params = params or {}
+        return self.request(endpoint, 'POST', params=params, version=version)
+
+    def delete(self, endpoint, params=None, version=1):
+        params = params or {}
+        return self.request(endpoint, 'DELETE', params=params, version=version)
+
+    # End Dynamic Request Methods
+
+    def get_lastfunction_header(self, header):
+        """
+            get_lastfunction_header(self)
+
+            returns the header in the last function
+            this must be called after an API call, as it returns header based information.
+            this will return None if the header is not present
+
+            most useful for the following header information:
+                x-ratelimit-limit
+                x-ratelimit-remaining
+                x-ratelimit-class
+                x-ratelimit-reset
+        """
+        if self._last_call is None:
+            raise TwythonError('This function must be called after an API call.  It delivers header information.')
+        if header in self._last_call['headers']:
+            return self._last_call['headers'][header]
+        return None
 
     def get_authentication_tokens(self):
         """
@@ -238,10 +334,12 @@ class Twython(object):
 
             Returns an authorization URL for a user to hit.
         """
-        callback_url = self.callback_url or 'oob'
+        callback_url = self.callback_url
 
         request_args = {}
-        request_args['oauth_callback'] = callback_url
+        if callback_url:
+            request_args['oauth_callback'] = callback_url
+
         method = 'get'
 
         func = getattr(self.client, method)
@@ -256,17 +354,12 @@ class Twython(object):
 
         oauth_callback_confirmed = request_tokens.get('oauth_callback_confirmed') == 'true'
 
-        if not OAUTH_LIB_SUPPORTS_CALLBACK and callback_url != 'oob' and oauth_callback_confirmed:
-            import warnings
-            warnings.warn("oauth2 library doesn't support OAuth 1.0a type callback, but remote requires it")
-            oauth_callback_confirmed = False
-
         auth_url_params = {
             'oauth_token': request_tokens['oauth_token'],
         }
 
-        # Use old-style callback argument
-        if OAUTH_CALLBACK_IN_URL or (callback_url != 'oob' and not oauth_callback_confirmed):
+        # Use old-style callback argument if server didn't accept new-style
+        if callback_url and not oauth_callback_confirmed:
             auth_url_params['oauth_callback'] = callback_url
 
         request_tokens['auth_url'] = self.authenticate_url + '?' + urllib.urlencode(auth_url_params)
